# coding=utf-8

import unittest

from conans.client.conf import default_settings_yml
from conans.model.editable_cpp_info import EditableCppInfo
from conans.model.settings import Settings


class WorkOnItemsTest(unittest.TestCase):

    def test_empty(self):
        self.assertEqual("", EditableCppInfo._work_on_item("", None, None))

    def test_placeholders(self):
        settings = Settings.loads(default_settings_yml)
        settings.compiler = 'Visual Studio'
        settings.compiler.version = '14'
        settings.build_type = 'Debug'

        self.assertEqual('src/Visual Studio14/Debug/include',
                         EditableCppInfo._work_on_item("src/{settings.compiler}{settings.compiler.version}/{settings.build_type}/include",
                                                       settings=settings,
                                                       options=None))
        self.assertEqual('C:/Visual Studio/include/',
                         EditableCppInfo._work_on_item("C:\\{settings.compiler}\\include\\",
                                                       settings=settings,
                                                       options=None))
        self.assertEqual('C:/Visual Studio/include/',
                         EditableCppInfo._work_on_item("C:\{settings.compiler}\include\\",
                                                       settings=settings,
                                                       options=None))
        self.assertEqual('/usr/path with spaces/Visual Studio/dir',
                         EditableCppInfo._work_on_item("/usr/path with spaces/{settings.compiler}/dir",
                                                       settings=settings,
                                                       options=None))



from six.moves.urllib.parse import urlencode

from conans.errors import ConanException
from conans.model.ref import ConanFileReference
from conans.model.rest_routes import RestRoutes
from conans.paths import CONAN_MANIFEST, CONANINFO


class ClientBaseRouterBuilder(object):
    bad_package_revision = "It is needed to specify the recipe revision if you " \
                           "specify a package revision"

    def __init__(self, base_url):
        self.routes = RestRoutes(base_url)

    def ping(self):
        return self.routes.ping

    @staticmethod
    def format_ref(url, ref):
        url = url.format(name=ref.name, version=ref.version, username=ref.user,
                         channel=ref.channel, revision=ref.revision)
        return url

    @staticmethod
    def format_ref_path(url, ref, path):
        url = url.format(name=ref.name, version=ref.version, username=ref.user,
                         channel=ref.channel, revision=ref.revision, path=path)
        return url

    @staticmethod
    def format_pref(url, pref):
        ref = pref.ref
        url = url.format(name=ref.name, version=ref.version, username=ref.user,
                         channel=ref.channel, revision=ref.revision, package_id=pref.id,
                         p_revision=pref.revision)
        return url

    @staticmethod
    def format_pref_path(url, pref, path):
        ref = pref.ref
        url = url.format(name=ref.name, version=ref.version, username=ref.user,
                         channel=ref.channel, revision=ref.revision, package_id=pref.id,
                         p_revision=pref.revision, path=path)
        return url

    def for_recipe(self, ref):
        """url for a recipe with or without revisions"""
        if not ref.revision:
            tmp = self.routes.recipe
        else:
            tmp = self.routes.recipe_revision
        return self.format_ref(tmp, ref)

    def for_packages(self, ref):
        """url for a recipe with or without revisions"""
        if not ref.revision:
            tmp = self.routes.packages
        else:
            tmp = self.routes.packages_revision
        return self.format_ref(tmp, ref)

    def for_recipe_file(self, ref, path):
        """url for a recipe file, with or without revisions"""
        if not ref.revision:
            tmp = self.routes.recipe_file
        else:
            tmp = self.routes.recipe_revision_file
        return self.format_ref_path(tmp, ref, path)

    def for_recipe_files(self, ref):
        """url for getting the recipe list"""
        if not ref.revision:
            tmp = self.routes.recipe_files
        else:
            tmp = self.routes.recipe_revision_files
        return self.format_ref(tmp, ref)

    def for_package(self, pref):
        """url for the package with or without revisions"""
        if not pref.ref.revision:
            if pref.revision:
                raise ConanException(self.bad_package_revision)
            tmp = self.routes.package
        elif not pref.revision:
            tmp = self.routes.package_recipe_revision
        else:
            tmp = self.routes.package_revision

        return self.format_pref(tmp, pref)

    def for_package_file(self, pref, path):
        """url for getting a file from a package, with or without revisions"""
        if not pref.ref.revision:
            if pref.revision:
                raise ConanException(self.bad_package_revision)
            tmp = self.routes.package_file
        elif not pref.revision:
            tmp = self.routes.package_recipe_revision_file
        else:
            tmp = self.routes.package_revision_file

        return self.format_pref_path(tmp, pref, path)

    def for_package_files(self, pref):
        """url for getting the recipe list"""
        if not pref.ref.revision:
            if pref.revision:
                raise ConanException(self.bad_package_revision)
            tmp = self.routes.package_files
        elif not pref.revision:
            tmp = self.routes.package_recipe_revision_files
        else:
            tmp = self.routes.package_revision_files

        return self.format_pref(tmp, pref)


class ClientSearchRouterBuilder(ClientBaseRouterBuilder):
    """Search urls shared between v1 and v2"""

    def __init__(self, base_url):
        super(ClientSearchRouterBuilder, self).__init__(base_url + "/conans")

    def search(self, pattern, ignorecase):
        """URL search recipes"""
        query = ''
        if pattern:
            if isinstance(pattern, ConanFileReference):
                pattern = pattern.full_repr()
            params = {"q": pattern}
            if not ignorecase:
                params["ignorecase"] = "False"
            query = "?%s" % urlencode(params)
        return "%s%s" % (self.routes.common_search, query)

    def search_packages(self, ref, query=None):
        """URL search packages for a recipe"""
        route = self.routes.common_search_packages_revision \
            if ref.revision else self.routes.common_search_packages
        url = self.format_ref(route, ref)
        if query:
            url += "?%s" % urlencode({"q": query})
        return url


class ClientUsersRouterBuilder(ClientBaseRouterBuilder):
    """Builds urls for users endpoint (shared v1 and v2)"""
    def __init__(self, base_url):
        super(ClientUsersRouterBuilder, self).__init__(base_url + "/users")

    def common_authenticate(self):
        return self.routes.common_authenticate

    def common_check_credentials(self):
        return self.routes.common_check_credentials


class ClientV1ConanRouterBuilder(ClientBaseRouterBuilder):
    """Builds urls for v1"""

    def __init__(self, base_url):
        super(ClientV1ConanRouterBuilder, self).__init__(base_url + "/conans")

    def remove_recipe(self, ref):
        """Remove recipe"""
        return self.for_recipe(ref.copy_clear_rev())

    def remove_recipe_files(self, ref):
        """Removes files from the recipe"""
        return self.format_ref(self.routes.v1_remove_recipe_files, ref.copy_clear_rev())

    def remove_packages(self, ref):
        """Remove files from a package"""
        return self.format_ref(self.routes.v1_remove_packages, ref.copy_clear_rev())

    def recipe_snapshot(self, ref):
        """get recipe manifest url"""
        return self.for_recipe(ref.copy_clear_rev())

    def package_snapshot(self, pref):
        """get recipe manifest url"""
        return self.for_package(pref.copy_clear_rev())

    def recipe_manifest(self, ref):
        """get recipe manifest url"""
        return self.format_ref(self.routes.v1_recipe_digest, ref.copy_clear_rev())

    def package_manifest(self, pref):
        """get manifest url"""
        return self.format_pref(self.routes.v1_package_digest, pref.copy_clear_rev())

    def recipe_download_urls(self, ref):
        """ urls to download the recipe"""
        return self.format_ref(self.routes.v1_recipe_download_urls, ref.copy_clear_rev())

    def package_download_urls(self, pref):
        """ urls to download the package"""
        return self.format_pref(self.routes.v1_package_download_urls, pref.copy_clear_rev())

    def recipe_upload_urls(self, ref):
        """ urls to upload the recipe"""
        return self.format_ref(self.routes.v1_recipe_upload_urls, ref.copy_clear_rev())

    def package_upload_urls(self, pref):
        """ urls to upload the package"""
        return self.format_pref(self.routes.v1_package_upload_urls, pref.copy_clear_rev())


class ClientV2ConanRouterBuilder(ClientBaseRouterBuilder):
    """Builds urls for v2"""

    def __init__(self, base_url):
        super(ClientV2ConanRouterBuilder, self).__init__(base_url + "/conans")

    def recipe_file(self, ref, path):
        """Recipe file url"""
        return self.for_recipe_file(ref, path)

    def package_file(self, pref, path):
        """Package file url"""
        return self.for_package_file(pref, path)

    def remove_recipe(self, ref):
        """Remove recipe url"""
        return self.for_recipe(ref)

    def remove_package(self, pref):
        """Remove package url"""
        return self.for_package(pref)

    def remove_all_packages(self, ref):
        """Remove package url"""
        return self.for_packages(ref)

    def recipe_manifest(self, ref):
        """Get the url for getting a conanmanifest.txt from a recipe"""
        return self.for_recipe_file(ref, CONAN_MANIFEST)

    def package_manifest(self, pref):
        """Get the url for getting a conanmanifest.txt from a package"""
        return self.for_package_file(pref, CONAN_MANIFEST)

    def package_info(self, pref):
        """Get the url for getting a conaninfo.txt from a package"""
        return self.for_package_file(pref, CONANINFO)

    def recipe_snapshot(self, ref):
        """get recipe manifest url"""
        return self.for_recipe_files(ref)

    def package_snapshot(self, pref):
        """get recipe manifest url"""
        return self.for_package_files(pref)



# coding=utf-8

import unittest

from conans.client.tools.scm import Git
from conans.test.utils.tools import temp_folder


class GitRemoteUrlTest(unittest.TestCase):

    def test_remove_credentials(self):
        """ Check that the 'remove_credentials' argument is taken into account """
        expected_url = 'https://myrepo.com/path/to/repo.git'
        origin_url = 'https://username:password@myrepo.com/path/to/repo.git'

        git = Git(folder=temp_folder())
        git.run("init .")
        git.run("remote add origin {}".format(origin_url))

        self.assertEqual(git.get_remote_url(), origin_url)
        self.assertEqual(git.get_remote_url(remove_credentials=True), expected_url)



# coding=utf-8
import os
from collections import OrderedDict

from six.moves import configparser

from conans.errors import ConanException
from conans.model.ref import ConanFileReference

DEFAULT_LAYOUT_FILE = "default"
LAYOUTS_FOLDER = 'layouts'


def get_editable_abs_path(path, cwd, cache_folder):
    # Check the layout file exists, is correct, and get its abs-path
    if path:
        layout_abs_path = path if os.path.isabs(path) else os.path.normpath(os.path.join(cwd, path))
        if not os.path.isfile(layout_abs_path):
            layout_abs_path = os.path.join(cache_folder, LAYOUTS_FOLDER, path)
        if not os.path.isfile(layout_abs_path):
            raise ConanException("Couldn't find layout file: %s" % path)
        EditableCppInfo.load(layout_abs_path)  # Try if it loads ok
        return layout_abs_path

    # Default only in cache
    layout_abs_path = os.path.join(cache_folder, LAYOUTS_FOLDER, DEFAULT_LAYOUT_FILE)
    if os.path.isfile(layout_abs_path):
        EditableCppInfo.load(layout_abs_path)
        return layout_abs_path


class EditableCppInfo(object):
    cpp_info_dirs = ['includedirs', 'libdirs', 'resdirs', 'bindirs', 'builddirs', 'srcdirs']

    def __init__(self, data):
        self._data = data

    @staticmethod
    def load(filepath):
        parser = configparser.ConfigParser(allow_no_value=True)
        parser.optionxform = str
        try:
            parser.read(filepath)
        except configparser.Error as e:
            raise ConanException("Error parsing layout file: %s\n%s" % (filepath, str(e)))
        data = OrderedDict()
        for section in parser.sections():
            ref, cpp_info_dir = section.rsplit(":", 1) if ':' in section else (None, section)
            if cpp_info_dir not in EditableCppInfo.cpp_info_dirs:
                raise ConanException("Wrong cpp_info field '%s' in layout file: %s"
                                     % (cpp_info_dir, filepath))
            if ref:
                try:
                    r = ConanFileReference.loads(ref)
                    if r.revision:
                        raise ConanException
                except ConanException:
                    raise ConanException("Wrong package reference '%s' in layout file: %s"
                                         % (ref, filepath))
            data.setdefault(ref, {})[cpp_info_dir] = [k for k, _ in parser.items(section)]

        return EditableCppInfo(data)

    @staticmethod
    def _work_on_item(value, settings, options):
        value = value.format(settings=settings, options=options)
        value = value.replace('\\', '/')
        return value

    def apply_to(self, ref, cpp_info, settings=None, options=None):
        d = self._data
        data = d.get(str(ref)) or d.get(None) or {}

        if data:  # Invalidate previously existing dirs
            for info_dir in self.cpp_info_dirs:
                setattr(cpp_info, info_dir, [])
        try:
            for key, items in data.items():
                setattr(cpp_info, key, [self._work_on_item(item, settings, options)
                                        for item in items])
        except Exception as e:
            raise ConanException("Error applying layout in '%s': %s" % (str(ref), str(e)))



from bottle import response

from conans.server.rest.controllers.controller import Controller


class PingController(Controller):

    def attach_to(self, app):

        @app.route("%s/ping" % self.route, method=["GET"])
        def ping():
            """
            Response OK. Useful to get server capabilities
            """
            response.headers['X-Conan-Server-Capabilities'] = ",".join(app.server_capabilities)
            return



import os
import platform
import unittest

from nose.plugins.attrib import attr
from parameterized import parameterized

from conans.client import tools
from conans.model.ref import PackageReference
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient
from conans.test.utils.visual_project_files import get_vs_project_files


class MSBuildTest(unittest.TestCase):

    @attr('slow')
    @unittest.skipUnless(platform.system() == "Windows", "Requires MSBuild")
    def build_vs_project_test(self):
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler", "cppstd"

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", verbosity="normal")

    def package(self):
        self.copy(pattern="*.exe")
"""
        client = TestClient()

        # Test cpp standard stuff

        files = get_vs_project_files(std="cpp17_2015")
        files[CONANFILE] = conan_build_vs

        client.save(files)
        client.run('create . Hello/1.2.1@lasote/stable -s cppstd=11 -s '
                   'compiler="Visual Studio" -s compiler.version=14', assert_error=True)
        client.run('create . Hello/1.2.1@lasote/stable -s cppstd=17 '
                   '-s compiler="Visual Studio" -s compiler.version=14')
        self.assertIn("Copied 1 '.exe' file: MyProject.exe", client.out)

        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs

        # Try to not update the project
        client.cache._conan_config = None  # Invalidate cached config
        tools.replace_in_file(client.cache.conan_conf_path, "[general]",
                              "[general]\nskip_vs_projects_upgrade = True")
        client.save(files, clean_first=True)
        client.run("create . Hello/1.2.1@lasote/stable --build")
        self.assertNotIn("devenv", client.user_io.out)
        self.assertIn("Skipped sln project upgrade", client.out)

        # Try with x86_64
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86_64")
        self.assertIn("Release|x64", client.user_io.out)
        self.assertIn("Copied 1 '.exe' file: MyProject.exe", client.out)

        # Try with x86
        client.save(files, clean_first=True)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86")
        self.assertIn("Release|x86", client.user_io.out)
        self.assertIn("Copied 1 '.exe' file: MyProject.exe", client.out)

        # Try with x86 debug
        client.save(files, clean_first=True)
        client.run("export . lasote/stable")
        client.run("install Hello/1.2.1@lasote/stable --build -s arch=x86 -s build_type=Debug")
        self.assertIn("Debug|x86", client.user_io.out)
        self.assertIn("Copied 1 '.exe' file: MyProject.exe", client.out)

        # Try with a custom property file name
        files[CONANFILE] = conan_build_vs.replace(
                'msbuild.build("MyProject.sln", verbosity="normal")',
                'msbuild.build("MyProject.sln", verbosity="normal", property_file_name="mp.props")')
        client.save(files, clean_first=True)
        client.run("create . Hello/1.2.1@lasote/stable --build -s arch=x86 -s build_type=Debug")
        self.assertIn("Debug|x86", client.user_io.out)
        self.assertIn("Copied 1 '.exe' file: MyProject.exe", client.out)
        full_ref = "Hello/1.2.1@lasote/stable:b786e9ece960c3a76378ca4d5b0d0e922f4cedc1"
        pref = PackageReference.loads(full_ref)
        build_folder = client.cache.build(pref)
        self.assertTrue(os.path.exists(os.path.join(build_folder, "mp.props")))

    @unittest.skipUnless(platform.system() == "Windows", "Requires MSBuild")
    def reuse_msbuild_object_test(self):
        # https://github.com/conan-io/conan/issues/2865
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler", "cppstd"

    def configure(self):
        del self.settings.compiler.runtime
        del self.settings.build_type

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", build_type="Release")
        msbuild.build("MyProject.sln", build_type="Debug")
        self.output.info("build() completed")
"""
        client = TestClient()
        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs

        client.save(files)
        client.run("create . danimtb/testing")
        self.assertIn("build() completed", client.out)

    @parameterized.expand([("True",), ("'my_log.binlog'",)])
    @unittest.skipUnless(platform.system() == "Windows", "Requires MSBuild")
    def binary_log_build_test(self, value):
        conan_build_vs = """
from conans import ConanFile, MSBuild

class HelloConan(ConanFile):
    name = "Hello"
    version = "1.2.1"
    exports = "*"
    settings = "os", "build_type", "arch", "compiler"

    def build(self):
        msbuild = MSBuild(self)
        msbuild.build("MyProject.sln", output_binary_log=%s)
"""
        client = TestClient()
        files = get_vs_project_files()
        files[CONANFILE] = conan_build_vs % value
        client.save(files)
        client.run("install . -s compiler=\"Visual Studio\" -s compiler.version=15")
        client.run("build .")

        if value == "'my_log.binlog'":
            log_name = value[1:1]
            flag = "/bl:%s" % log_name
        else:
            log_name = "msbuild.binlog"
            flag = "/bl"

        self.assertIn(flag, client.out)
        log_path = os.path.join(client.current_folder, log_name)
        self.assertTrue(os.path.exists(log_path))



# coding=utf-8

import os
import platform
from contextlib import contextmanager

from conans.model.manifest import FileTreeManifest
from conans.model.package_metadata import PackageMetadata
from conans.model.ref import ConanFileReference
from conans.model.ref import PackageReference
from conans.paths import CONANFILE, SYSTEM_REQS, EXPORT_FOLDER, EXPORT_SRC_FOLDER, SRC_FOLDER, \
    BUILD_FOLDER, PACKAGES_FOLDER, SYSTEM_REQS_FOLDER, SCM_FOLDER, PACKAGE_METADATA
from conans.util.files import load, save


def short_path(func):
    if platform.system() == "Windows":
        from conans.util.windows import path_shortener

        def wrap(self, *args, **kwargs):
            p = func(self,  *args, **kwargs)
            return path_shortener(p, self._short_paths)
        return wrap
    else:
        return func


class PackageCacheLayout(object):
    """ This is the package layout for Conan cache """

    def __init__(self, base_folder, ref, short_paths):
        assert isinstance(ref, ConanFileReference)
        self._ref = ref
        self._base_folder = os.path.normpath(base_folder)
        self._short_paths = short_paths

    def conan(self):
        """ Returns the base folder for this package reference """
        return self._base_folder

    def export(self):
        return os.path.join(self.conan(), EXPORT_FOLDER)

    @short_path
    def export_sources(self):
        return os.path.join(self.conan(), EXPORT_SRC_FOLDER)

    @short_path
    def source(self):
        return os.path.join(self.conan(), SRC_FOLDER)

    def conanfile(self):
        export = self.export()
        return os.path.join(export, CONANFILE)

    def builds(self):
        return os.path.join(self.conan(), BUILD_FOLDER)

    @short_path
    def build(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref
        return os.path.join(self.conan(), BUILD_FOLDER, pref.id)

    def system_reqs(self):
        return os.path.join(self.conan(), SYSTEM_REQS_FOLDER, SYSTEM_REQS)

    def system_reqs_package(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref
        return os.path.join(self.conan(), SYSTEM_REQS_FOLDER, pref.id, SYSTEM_REQS)

    def packages(self):
        return os.path.join(self.conan(), PACKAGES_FOLDER)

    @short_path
    def package(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref
        return os.path.join(self.conan(), PACKAGES_FOLDER, pref.id)

    def scm_folder(self):
        return os.path.join(self.conan(), SCM_FOLDER)

    def package_metadata(self):
        return os.path.join(self.conan(), PACKAGE_METADATA)

    def load_manifest(self):
        return FileTreeManifest.load(self.export())

    def package_manifests(self, package_reference):
        package_folder = self.package(package_reference)
        readed_manifest = FileTreeManifest.load(package_folder)
        expected_manifest = FileTreeManifest.create(package_folder)
        return readed_manifest, expected_manifest

    # Metadata
    def load_metadata(self):
        try:
            text = load(self.package_metadata())
            return PackageMetadata.loads(text)
        except IOError:
            return PackageMetadata()

    @contextmanager
    def update_metadata(self):
        metadata = self.load_metadata()
        yield metadata
        save(self.package_metadata(), metadata.dumps())

    # Revisions
    def package_summary_hash(self, pref):
        package_folder = self.package(pref)
        readed_manifest = FileTreeManifest.load(package_folder)
        return readed_manifest.summary_hash



# coding=utf-8

import unittest

from mock import patch

from conans.client.tools.scm import SVN
from conans.test.utils.tools import temp_folder


class SVNRemoteUrlTest(unittest.TestCase):

    def test_remove_credentials(self):
        """ Check that the 'remove_credentials' argument is taken into account """
        expected_url = 'https://myrepo.com/path/to/repo'
        origin_url = 'https://username:password@myrepo.com/path/to/repo'

        svn = SVN(folder=temp_folder())

        # Mocking, as we cannot change SVN remote to a non-existing url
        with patch.object(svn, '_show_item', return_value=origin_url):
            self.assertEqual(svn.get_remote_url(), origin_url)
            self.assertEqual(svn.get_remote_url(remove_credentials=True), expected_url)



import os

from conans.client import packager
from conans.client.graph.graph_manager import load_deps_info
from conans.errors import ConanException
from conans.model.ref import PackageReference
from conans.util.files import rmdir


def export_pkg(cache, graph_manager, hook_manager, recorder, output,
               ref, source_folder, build_folder, package_folder, install_folder,
               graph_info, force):

    conan_file_path = cache.conanfile(ref)
    if not os.path.exists(conan_file_path):
        raise ConanException("Package recipe '%s' does not exist" % str(ref))

    deps_graph = graph_manager.load_simple_graph(ref, graph_info.profile, recorder)

    # this is a bit tricky, but works. The root (virtual), has only 1 neighbor,
    # which is the exported pkg
    nodes = deps_graph.root.neighbors()
    conanfile = nodes[0].conanfile
    from conans.client.conan_api import existing_info_files
    if install_folder and existing_info_files(install_folder):
        load_deps_info(install_folder, conanfile, required=True)
    package_id = conanfile.info.package_id()
    output.info("Packaging to %s" % package_id)
    pref = PackageReference(ref, package_id)
    dest_package_folder = cache.package(pref, short_paths=conanfile.short_paths)

    if os.path.exists(dest_package_folder):
        if force:
            rmdir(dest_package_folder)
        else:
            raise ConanException("Package already exists. Please use --force, -f to "
                                 "overwrite it")

    recipe_hash = cache.package_layout(ref).load_manifest().summary_hash
    conanfile.info.recipe_hash = recipe_hash
    conanfile.develop = True
    if package_folder:
        packager.export_pkg(conanfile, package_id, package_folder, dest_package_folder,
                            hook_manager, conan_file_path, ref)
    else:
        packager.create_package(conanfile, package_id, source_folder, build_folder,
                                dest_package_folder, install_folder, hook_manager, conan_file_path,
                                ref, local=True)
    recorder.package_exported(pref)



from conans.client.output import ScopedOutput
from conans.client.source import complete_recipe_sources
from conans.errors import NotFoundException
from conans.model.ref import ConanFileReference, PackageReference


def download(ref, package_ids, remote_name, recipe, remote_manager,
             cache, out, recorder, loader, hook_manager):

    assert(isinstance(ref, ConanFileReference))
    output = ScopedOutput(str(ref), out)
    registry = cache.registry
    remote = registry.remotes.get(remote_name) if remote_name else registry.remotes.default

    hook_manager.execute("pre_download", reference=ref, remote=remote)
    # First of all download package recipe
    try:
        remote_manager.get_recipe(ref, remote)
    except NotFoundException:
        raise NotFoundException("'%s' not found in remote '%s'" % (str(ref), remote.name))
    registry.refs.set(ref, remote.name)
    conan_file_path = cache.conanfile(ref)
    conanfile = loader.load_class(conan_file_path)

    if not recipe:  # Not only the recipe
        # Download the sources too, don't be lazy
        complete_recipe_sources(remote_manager, cache, conanfile, ref)

        if not package_ids:  # User didnt specify a specific package binary
            output.info("Getting the complete package list from '%s'..." % str(ref))
            packages_props = remote_manager.search_packages(remote, ref, None)
            package_ids = list(packages_props.keys())
            if not package_ids:
                output.warn("No remote binary packages found in remote")

        _download_binaries(conanfile, ref, package_ids, cache, remote_manager,
                           remote, output, recorder)
    hook_manager.execute("post_download", conanfile_path=conan_file_path, reference=ref,
                         remote=remote)


def _download_binaries(conanfile, ref, package_ids, cache, remote_manager, remote, output, recorder):
    short_paths = conanfile.short_paths

    for package_id in package_ids:
        pref = PackageReference(ref, package_id)
        package_folder = cache.package(pref, short_paths=short_paths)
        output.info("Downloading %s" % str(pref))
        remote_manager.get_package(pref, package_folder, remote, output, recorder)



import os
import re
from fnmatch import translate

import jwt

from conans import load
from conans.errors import ConanException, ForbiddenException, NotFoundException, \
    RequestErrorException
from conans.model.info import ConanInfo
from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONANINFO
from conans.search.search import _partial_match, filter_packages
from conans.util.files import list_folder_subdirs, mkdir
from conans.util.log import logger


class FileUploadDownloadService(object):
    """Handles authorization from token and upload and download files"""

    def __init__(self, updown_auth_manager, base_store_folder):
        self.updown_auth_manager = updown_auth_manager
        self.base_store_folder = base_store_folder

    def get_file_path(self, filepath, token):
        try:
            encoded_path, _, user = self.updown_auth_manager.get_resource_info(token)
            if not self._valid_path(filepath, encoded_path):
                logger.info("Invalid path file!! %s: %s" % (user, filepath))
                raise NotFoundException("File not found")
            logger.debug("Get file: user=%s path=%s" % (user, filepath))
            file_path = os.path.normpath(os.path.join(self.base_store_folder, encoded_path))
            return file_path
        except (jwt.ExpiredSignature, jwt.DecodeError, AttributeError):
            raise NotFoundException("File not found")

    def put_file(self, file_saver, abs_filepath, token, upload_size):
        """
        file_saver is an object with the save() method without parameters
        """
        try:
            encoded_path, filesize, user = self.updown_auth_manager.get_resource_info(token)
            # Check size
            if upload_size != filesize:
                logger.debug("Invalid size file!!: %s: %s" % (user, abs_filepath))
                raise RequestErrorException("Bad file size")

            abs_encoded_path = os.path.abspath(os.path.join(self.base_store_folder, encoded_path))
            if not self._valid_path(abs_filepath, abs_encoded_path):
                raise NotFoundException("File not found")
            logger.debug("Put file: %s: %s" % (user, abs_filepath))
            mkdir(os.path.dirname(abs_filepath))
            if os.path.exists(abs_filepath):
                os.remove(abs_filepath)
            file_saver.save(os.path.dirname(abs_filepath))

        except (jwt.ExpiredSignature, jwt.DecodeError, AttributeError):
            raise NotFoundException("File not found")

    def _valid_path(self, filepath, encoded_path):
        if encoded_path == filepath:
            path = os.path.join(self.base_store_folder, encoded_path)
            path = os.path.normpath(path)
            # Protect from path outside storage "../.."
            if not path.startswith(self.base_store_folder):
                return False
            return True
        else:
            return False


class SearchService(object):

    def __init__(self, authorizer, server_store, auth_user):
        self._authorizer = authorizer
        self._server_store = server_store
        self._auth_user = auth_user

    def search_packages(self, reference, query, v2_compatibility_mode):
        self._authorizer.check_read_conan(self._auth_user, reference)
        info = search_packages(self._server_store, reference, query, v2_compatibility_mode)
        return info

    def search_recipes(self, pattern=None, ignorecase=True):

        def get_ref(_pattern):
            if not isinstance(_pattern, ConanFileReference):
                try:
                    ref_ = ConanFileReference.loads(_pattern)
                except (ConanException, TypeError):
                    ref_ = None
            else:
                ref_ = _pattern
            return ref_

        # Check directly if it is a reference
        ref = get_ref(pattern)
        if ref:
            # Avoid resolve latest revision if a version range is passed or we are performing a
            # package remove (all revisions)
            path = self._server_store.conan(ref, resolve_latest=False)
            if self._server_store.path_exists(path):
                return [ref]

        # Conan references in main storage
        if pattern:
            pattern = str(pattern)
            b_pattern = translate(pattern)
            b_pattern = re.compile(b_pattern, re.IGNORECASE) if ignorecase else re.compile(b_pattern)

        subdirs = list_folder_subdirs(basedir=self._server_store.store, level=5)
        if not pattern:
            return sorted([ConanFileReference(*folder.split("/")) for folder in subdirs])
        else:
            ret = []
            for subdir in subdirs:
                ref = ConanFileReference(*subdir.split("/"))
                if _partial_match(b_pattern, ref):
                    ret.append(ref)

            return sorted(ret)

    def search(self, pattern=None, ignorecase=True):
        """ Get all the info about any package
            Attributes:
                pattern = wildcards like opencv/*
        """
        refs = self.search_recipes(pattern, ignorecase)
        filtered = []
        # Filter out restricted items
        for ref in refs:
            try:
                self._authorizer.check_read_conan(self._auth_user, ref)
                filtered.append(ref)
            except ForbiddenException:
                pass
        return filtered


class ConanService(object):
    """Handles authorization and expose methods for REST API"""

    def __init__(self, authorizer, server_store, auth_user):

        self._authorizer = authorizer
        self._server_store = server_store
        self._auth_user = auth_user

    def get_recipe_snapshot(self, ref):
        """Gets a dict with filepaths and the md5:
            {filename: md5}
        """
        self._authorizer.check_read_conan(self._auth_user, ref)
        snap = self._server_store.get_recipe_snapshot(ref)
        if not snap:
            raise NotFoundException("conanfile not found")
        return snap

    def get_conanfile_download_urls(self, ref, files_subset=None):
        """Gets a dict with filepaths and the urls:
            {filename: url}
        """
        self._authorizer.check_read_conan(self._auth_user, ref)
        urls = self._server_store.get_download_conanfile_urls(ref, files_subset, self._auth_user)
        if not urls:
            raise NotFoundException("conanfile not found")
        return urls

    def get_conanfile_upload_urls(self, ref, filesizes):
        _validate_conan_reg_filenames(list(filesizes.keys()))
        self._authorizer.check_write_conan(self._auth_user, ref)
        urls = self._server_store.get_upload_conanfile_urls(ref, filesizes, self._auth_user)
        return urls

    def remove_conanfile(self, ref):
        self._authorizer.check_delete_conan(self._auth_user, ref)
        self._server_store.remove_conanfile(ref)

    def remove_packages(self, ref, package_ids_filter):
        """If the revision is not specified it will remove the packages from all the recipes
        (v1 compatibility)"""
        for package_id in package_ids_filter:
            pref = PackageReference(ref, package_id)
            self._authorizer.check_delete_package(self._auth_user, pref)
        if not package_ids_filter:  # Remove all packages, check that we can remove conanfile
            self._authorizer.check_delete_conan(self._auth_user, ref)

        if ref.revision:
            references = [ref]
        else:
            references = self._server_store.get_recipe_revisions(ref)

        for ref in references:
            self._server_store.remove_packages(ref, package_ids_filter)

    def remove_package(self, pref):
        self._authorizer.check_delete_package(self._auth_user, pref)

        if not pref.ref.revision:
            recipe_revisions = self._server_store.get_recipe_revisions(pref.ref)
        else:
            recipe_revisions = [pref.ref, ]

        for ref in recipe_revisions:
            if not pref.revision:
                pref = PackageReference(ref, pref.id)
                package_revisions = [r.revision
                                     for r in self._server_store.get_package_revisions(pref)]
            else:
                package_revisions = [pref.revision]

            for prev in package_revisions:
                full_pref = PackageReference(ref, pref.id, prev)
                self._server_store.remove_package(full_pref)

    def remove_all_packages(self, ref):
        if ref.revision:
            refs = [ref]
        else:
            refs = self._server_store.get_recipe_revisions(ref)
        for ref in refs:
            self._server_store.remove_all_packages(ref)

    def remove_conanfile_files(self, ref, files):
        self._authorizer.check_delete_conan(self._auth_user, ref)
        self._server_store.remove_conanfile_files(ref, files)

    def remove_conanfile_file(self, ref, path):
        self.remove_conanfile_files(ref, [path])

    # Package methods
    def get_package_snapshot(self, pref):
        """Gets a list with filepaths and the urls and md5:
            [filename: {'url': url, 'md5': md5}]
        """
        self._authorizer.check_read_package(self._auth_user, pref)
        snap = self._server_store.get_package_snapshot(pref)
        return snap

    def get_package_download_urls(self, pref, files_subset=None):
        """Gets a list with filepaths and the urls and md5:
            [filename: {'url': url, 'md5': md5}]
        """
        self._authorizer.check_read_package(self._auth_user, pref)
        urls = self._server_store.get_download_package_urls(pref, files_subset=files_subset)
        return urls

    def get_package_upload_urls(self, pref, filesizes):
        """
        :param pref: PackageReference
        :param filesizes: {filepath: bytes}
        :return {filepath: url} """
        try:
            self._server_store.get_recipe_snapshot(pref.ref)
        except NotFoundException:
            raise NotFoundException("There are no remote conanfiles like %s" % str(pref.ref))
        self._authorizer.check_write_package(self._auth_user, pref)
        urls = self._server_store.get_upload_package_urls(pref, filesizes, self._auth_user)
        return urls


def _validate_conan_reg_filenames(files):
    message = "Invalid conans request"

# Could be partial uploads, so we can't expect for all files to be present
#     # conanfile and digest in files
#     if CONANFILE not in files:
#         # Log something
#         raise RequestErrorException("Missing %s" % CONANFILE)
#     if CONAN_MANIFEST not in files:
#         # Log something
#         raise RequestErrorException("Missing %s" % CONAN_MANIFEST)

    # All contents in same directory (from conan_id)
    for filename in files:
        if ".." in filename:
            # Log something
            raise RequestErrorException(message)


def search_packages(paths, ref, query, v2_compatibility_mode):
    """ Return a dict like this:

            {package_ID: {name: "OpenCV",
                           version: "2.14",
                           settings: {os: Windows}}}
    param ref: ConanFileReference object
    """
    if not os.path.exists(paths.conan(ref)):
        raise NotFoundException("Recipe not found: %s" % str(ref))
    infos = _get_local_infos_min(paths, ref, v2_compatibility_mode)
    return filter_packages(query, infos)


def _get_local_infos_min(paths, ref, v2_compatibility_mode=False):

    result = {}

    if not ref.revision and v2_compatibility_mode:
        recipe_revisions = paths.get_recipe_revisions(ref)
    else:
        recipe_revisions = [ref]

    for recipe_revision in recipe_revisions:
        packages_path = paths.packages(recipe_revision)
        subdirs = list_folder_subdirs(packages_path, level=1)
        for package_id in subdirs:
            if package_id in result:
                continue
            # Read conaninfo
            try:
                pref = PackageReference(ref, package_id)
                info_path = os.path.join(paths.package(pref, short_paths=None), CONANINFO)
                if not os.path.exists(info_path):
                    raise NotFoundException("")
                conan_info_content = load(info_path)
                info = ConanInfo.loads(conan_info_content)
                conan_vars_info = info.serialize_min()
                result[package_id] = conan_vars_info

            except Exception as exc:  # FIXME: Too wide
                logger.error("Package %s has no ConanInfo file" % str(pref))
                if str(exc):
                    logger.error(str(exc))
    return result



import os

import time

from conans.client.remote_manager import check_compressed_files
from conans.client.rest.client_routes import ClientV2ConanRouterBuilder
from conans.client.rest.rest_client_common import RestCommonMethods
from conans.client.rest.uploader_downloader import Downloader, Uploader
from conans.errors import ConanException, NotFoundException
from conans.model.info import ConanInfo
from conans.model.manifest import FileTreeManifest
from conans.model.ref import PackageReference, ConanFileReference
from conans.paths import EXPORT_SOURCES_TGZ_NAME, EXPORT_TGZ_NAME, \
    PACKAGE_TGZ_NAME
from conans.util.files import decode_text
from conans.util.log import logger


class RestV2Methods(RestCommonMethods):

    def __init__(self, remote_url, token, custom_headers, output, requester, verify_ssl,
                 put_headers=None, checksum_deploy=False):

        super(RestV2Methods, self).__init__(remote_url, token, custom_headers, output, requester,
                                            verify_ssl, put_headers)
        self._checksum_deploy = checksum_deploy

    @property
    def remote_api_url(self):
        return "%s/v2" % self.remote_url.rstrip("/")

    @property
    def conans_router(self):
        return ClientV2ConanRouterBuilder(self.remote_api_url)

    def _get_file_list_json(self, url):
        data = self.get_json(url)
        # Discarding (.keys()) still empty metadata for files
        data["files"] = list(data["files"].keys())
        return data

    def _get_remote_file_contents(self, url):
        downloader = Downloader(self.requester, self._output, self.verify_ssl)
        contents = downloader.download(url, auth=self.auth)
        return contents

    def _get_snapshot(self, url, reference):
        try:
            data = self._get_file_list_json(url)
            files_list = [os.path.normpath(filename) for filename in data["files"]]
            reference = data["reference"]
            rev_time = data["time"]
        except NotFoundException:
            files_list = []
            rev_time = None
        return files_list, reference, rev_time

    def _get_recipe_snapshot(self, ref):
        url = self.conans_router.recipe_snapshot(ref)
        ref_str = ref.full_repr()
        snap, reference, rev_time = self._get_snapshot(url, ref_str)
        ref = ConanFileReference.loads(reference)
        return snap, ref, rev_time

    def _get_package_snapshot(self, pref):
        url = self.conans_router.package_snapshot(pref)
        pref_str = pref.full_repr()
        snap, package_reference, rev_time = self._get_snapshot(url, pref_str)

        pref = PackageReference.loads(package_reference)
        return snap, pref, rev_time

    def get_conan_manifest(self, ref):
        url = self.conans_router.recipe_manifest(ref)
        content = self._get_remote_file_contents(url)
        return FileTreeManifest.loads(decode_text(content))

    def get_package_manifest(self, pref):
        url = self.conans_router.package_manifest(pref)
        content = self._get_remote_file_contents(url)
        return FileTreeManifest.loads(decode_text(content))

    def get_package_info(self, package_reference):
        url = self.conans_router.package_info(package_reference)
        content = self._get_remote_file_contents(url)
        return ConanInfo.loads(decode_text(content))

    def get_recipe(self, ref, dest_folder):
        url = self.conans_router.recipe_snapshot(ref)
        data = self._get_file_list_json(url)
        files = data["files"]
        rev_time = data["time"]
        check_compressed_files(EXPORT_TGZ_NAME, files)
        new_ref = ConanFileReference.loads(data["reference"])
        if EXPORT_SOURCES_TGZ_NAME in files:
            files.remove(EXPORT_SOURCES_TGZ_NAME)

        # If we didn't indicated reference, server got the latest, use absolute now, it's safer
        urls = {fn: self.conans_router.recipe_file(ref, fn) for fn in files}
        self._download_and_save_files(urls, dest_folder, files)
        ret = {fn: os.path.join(dest_folder, fn) for fn in files}
        return ret, new_ref, rev_time

    def get_recipe_sources(self, ref, dest_folder):
        url = self.conans_router.recipe_snapshot(ref)
        data = self._get_file_list_json(url)
        files = data["files"]
        check_compressed_files(EXPORT_SOURCES_TGZ_NAME, files)
        if EXPORT_SOURCES_TGZ_NAME not in files:
            return None
        files = [EXPORT_SOURCES_TGZ_NAME, ]

        # If we didn't indicated reference, server got the latest, use absolute now, it's safer
        new_ref = ConanFileReference.loads(data["reference"])
        urls = {fn: self.conans_router.recipe_file(new_ref, fn) for fn in files}
        self._download_and_save_files(urls, dest_folder, files)
        ret = {fn: os.path.join(dest_folder, fn) for fn in files}
        return ret

    def get_package(self, pref, dest_folder):
        url = self.conans_router.package_snapshot(pref)
        data = self._get_file_list_json(url)
        files = data["files"]
        rev_time = data["time"]
        check_compressed_files(PACKAGE_TGZ_NAME, files)
        # If we didn't indicated reference, server got the latest, use absolute now, it's safer
        pref = PackageReference.loads(data["reference"])
        urls = {fn: self.conans_router.package_file(pref, fn) for fn in files}
        self._download_and_save_files(urls, dest_folder, files)
        ret = {fn: os.path.join(dest_folder, fn) for fn in files}
        return ret, pref, rev_time

    def get_path(self, ref, package_id, path):

        if not package_id:
            url = self.conans_router.recipe_snapshot(ref)
        else:
            pref = PackageReference(ref, package_id)
            url = self.conans_router.package_snapshot(pref)

        try:
            files = self._get_file_list_json(url)
        except NotFoundException:
            if package_id:
                raise NotFoundException("Package %s:%s not found" % (ref, package_id))
            else:
                raise NotFoundException("Recipe %s not found" % str(ref))

        def is_dir(the_path):
            if the_path == ".":
                return True
            for the_file in files["files"]:
                if the_path == the_file:
                    return False
                elif the_file.startswith(the_path):
                    return True
            raise NotFoundException("The specified path doesn't exist")

        if is_dir(path):
            ret = []
            for the_file in files["files"]:
                if path == "." or the_file.startswith(path):
                    tmp = the_file[len(path)-1:].split("/", 1)[0]
                    if tmp not in ret:
                        ret.append(tmp)
            return sorted(ret)
        else:
            if not package_id:
                url = self.conans_router.recipe_file(ref, path)
            else:
                pref = PackageReference(ref, package_id)
                url = self.conans_router.package_file(pref, path)

            content = self._get_remote_file_contents(url)
            return decode_text(content)

    def _upload_recipe(self, ref, files_to_upload, retry, retry_wait):
        # Direct upload the recipe
        urls = {fn: self.conans_router.recipe_file(ref, fn) for fn in files_to_upload}
        self._upload_files(files_to_upload, urls, retry, retry_wait)

    def _upload_package(self, pref, files_to_upload, retry, retry_wait):
        urls = {fn: self.conans_router.package_file(pref, fn)
                for fn in files_to_upload}
        self._upload_files(files_to_upload, urls, retry, retry_wait)

    def _upload_files(self, files, urls, retry, retry_wait):
        t1 = time.time()
        failed = []
        uploader = Uploader(self.requester, self._output, self.verify_ssl)
        # Take advantage of filenames ordering, so that conan_package.tgz and conan_export.tgz
        # can be < conanfile, conaninfo, and sent always the last, so smaller files go first
        for filename in sorted(files, reverse=True):
            self._output.rewrite_line("Uploading %s" % filename)
            resource_url = urls[filename]
            try:
                response = uploader.upload(resource_url, files[filename], auth=self.auth,
                                           dedup=self._checksum_deploy, retry=retry,
                                           retry_wait=retry_wait,
                                           headers=self._put_headers)
                self._output.writeln("")
                if not response.ok:
                    self._output.error("\nError uploading file: %s, '%s'" % (filename,
                                                                             response.content))
                    failed.append(filename)
                else:
                    pass
            except Exception as exc:
                self._output.error("\nError uploading file: %s, '%s'" % (filename, exc))
                failed.append(filename)

        if failed:
            raise ConanException("Execute upload again to retry upload the failed files: %s"
                                 % ", ".join(failed))
        else:
            logger.debug("\nUPLOAD: All uploaded! Total time: %s\n" % str(time.time() - t1))

    def _download_and_save_files(self, urls, dest_folder, files):
        downloader = Downloader(self.requester, self._output, self.verify_ssl)
        # Take advantage of filenames ordering, so that conan_package.tgz and conan_export.tgz
        # can be < conanfile, conaninfo, and sent always the last, so smaller files go first
        for filename in sorted(files, reverse=True):
            if self._output:
                self._output.writeln("Downloading %s" % filename)
            resource_url = urls[filename]
            abs_path = os.path.join(dest_folder, filename)
            downloader.download(resource_url, abs_path, auth=self.auth)

    def _remove_conanfile_files(self, ref, files):
        # V2 === revisions, do not remove files, it will create a new revision if the files changed
        return

    def remove_packages(self, ref, package_ids=None):
        """ Remove any packages specified by package_ids"""
        self.check_credentials()
        if not package_ids:
            url = self.conans_router.remove_all_packages(ref)
            self.requester.delete(url, auth=self.auth, headers=self.custom_headers,
                                  verify=self.verify_ssl)
            return
        for pid in package_ids:
            pref = PackageReference(ref, pid)
            url = self.conans_router.remove_package(pref)
            self.requester.delete(url, auth=self.auth, headers=self.custom_headers,
                                  verify=self.verify_ssl)



import unittest

from conans import CMake
from conans.client.build.cmake_flags import get_generator
from conans.test.utils.conanfile import MockSettings, ConanFileMock


class CMakeGeneratorTest(unittest.TestCase):

    def test_cmake_default_generator_linux(self):
        settings = MockSettings({"os_build": "Linux"})
        generator = get_generator(settings)
        self.assertEquals("Unix Makefiles", generator)
        self._test_default_generator_cmake(settings, "Unix Makefiles")

    def test_cmake_default_generator_osx(self):
        settings = MockSettings({"os_build": "Macos"})
        generator = get_generator(settings)
        self.assertEquals("Unix Makefiles", generator)
        self._test_default_generator_cmake(settings, "Unix Makefiles")

    def test_default_generator_windows(self):
        settings = MockSettings({"os_build": "Windows"})
        generator = get_generator(settings)
        self.assertEquals("MinGW Makefiles", generator)
        self._test_default_generator_cmake(settings, "MinGW Makefiles")

    def _test_default_generator_cmake(self, settings, generator):
        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        self.assertEquals(generator, cmake.generator)
        self.assertIn('-G "{}"'.format(generator), cmake.command_line)




import os
import unittest
from collections import namedtuple

from conans.client.generators.cmake_paths import CMakePathsGenerator
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestBufferConanOutput


class CMakePathsGeneratorTest(unittest.TestCase):

    def cmake_vars_unit_test(self):
        settings_mock = namedtuple("Settings", "build_type, os, os_build, constraint")
        settings = settings_mock("Release", None, None, lambda x: x)
        conanfile = ConanFile(TestBufferConanOutput(), None)
        conanfile.initialize(settings, EnvValues())
        tmp = temp_folder()
        cpp_info = CppInfo(tmp)
        custom_dir = os.path.join(tmp, "custom_build_dir")
        os.mkdir(custom_dir)
        cpp_info.builddirs.append(os.path.join(tmp, "custom_build_dir"))
        conanfile.deps_cpp_info.update(cpp_info, "MyLib")

        generator = CMakePathsGenerator(conanfile)
        cmake_lines = [s.replace("\t\t\t", "").replace('\\', '/')
                       for s in generator.content.splitlines()]
        self.assertEquals('set(CMAKE_MODULE_PATH '
                          '"%s/"' % tmp.replace('\\', '/'), cmake_lines[0])
        self.assertEquals('"%s" ${CMAKE_MODULE_PATH} '
                          '${CMAKE_CURRENT_LIST_DIR})' % custom_dir.replace('\\', '/'),
                          cmake_lines[1])
        self.assertEquals('set(CMAKE_PREFIX_PATH '
                          '"%s/"' % tmp.replace('\\', '/'), cmake_lines[2])
        self.assertEquals('"%s" ${CMAKE_PREFIX_PATH} '
                          '${CMAKE_CURRENT_LIST_DIR})' % custom_dir.replace('\\', '/'),
                          cmake_lines[3])



import os
import textwrap
import unittest

from conans import Settings
from conans.client.graph.python_requires import ConanPythonRequire
from conans.client.loader import ConanFileLoader
from conans.model.env_info import EnvValues
from conans.model.profile import Profile
from conans.model.ref import ConanFileReference
from conans.test.utils.conanfile import MockSettings
from conans.test.utils.runner import TestRunner
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestBufferConanOutput
from conans.util.files import save


class LoadConanfileTxtTest(unittest.TestCase):

    def setUp(self):
        settings = Settings()
        self.profile = Profile()
        self.profile._settings = settings
        self.profile._user_options = None
        self.profile._env_values = None
        self.conanfile_txt_path = os.path.join(temp_folder(), "conanfile.txt")
        output = TestBufferConanOutput()
        self.loader = ConanFileLoader(TestRunner(output), output, None)

    def env_test(self):
        env_values = EnvValues()
        env_values.add("PREPEND_PATH", ["hello", "bye"])
        env_values.add("VAR", ["var_value"])
        self.profile._env_values = env_values
        save(self.conanfile_txt_path, "")
        conanfile = self.loader.load_conanfile_txt(self.conanfile_txt_path, self.profile)
        self.assertEquals(conanfile.env, {"PREPEND_PATH": ["hello", "bye"], "VAR": ["var_value"]})


class LoadConanfileTest(unittest.TestCase):

    def setUp(self):
        settings = Settings()
        self.profile = Profile()
        self.profile._settings = settings
        self.profile._user_options = None
        self.profile._env_values = None
        self.profile._dev_reference = None
        self.profile._package_settings = None
        self.conanfile_path = os.path.join(temp_folder(), "conanfile.py")
        output = TestBufferConanOutput()
        self.loader = ConanFileLoader(TestRunner(output), output, ConanPythonRequire(None, None))

    def env_test(self):
        env_values = EnvValues()
        env_values.add("PREPEND_PATH", ["hello", "bye"])
        env_values.add("VAR", ["var_value"])
        self.profile._env_values = env_values
        save(self.conanfile_path,
             textwrap.dedent("""
                from conans import ConanFile
                
                class TestConan(ConanFile):
                    name = "hello"
                    version = "1.0"
             """))
        ref = ConanFileReference("hello", "1.0", "user", "channel")
        conanfile = self.loader.load_conanfile(self.conanfile_path, self.profile, ref)
        self.assertEquals(conanfile.env, {"PREPEND_PATH": ["hello", "bye"], "VAR": ["var_value"]})



import os
import re
from fnmatch import translate

from conans.errors import ConanException, NotFoundException
from conans.model.info import ConanInfo
from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONANINFO
from conans.search.query_parse import evaluate_postfix, infix_to_postfix
from conans.util.files import list_folder_subdirs, load
from conans.util.log import logger


def filter_outdated(packages_infos, recipe_hash):
    result = {}
    for package_id, info in packages_infos.items():
        try:  # Existing package_info of old package might not have recipe_hash
            if info["recipe_hash"] != recipe_hash:
                result[package_id] = info
        except KeyError:
            pass
    return result


def filter_packages(query, package_infos):
    if query is None:
        return package_infos
    try:
        if "!" in query:
            raise ConanException("'!' character is not allowed")
        if " not " in query or query.startswith("not "):
            raise ConanException("'not' operator is not allowed")
        postfix = infix_to_postfix(query) if query else []
        result = {}
        for package_id, info in package_infos.items():
            if evaluate_postfix_with_info(postfix, info):
                result[package_id] = info
        return result
    except Exception as exc:
        raise ConanException("Invalid package query: %s. %s" % (query, exc))


def evaluate_postfix_with_info(postfix, conan_vars_info):

    # Evaluate conaninfo with the expression

    def evaluate_info(expression):
        """Receives an expression like compiler.version="12"
        Uses conan_vars_info in the closure to evaluate it"""
        name, value = expression.split("=", 1)
        value = value.replace("\"", "")
        return evaluate(name, value, conan_vars_info)

    return evaluate_postfix(postfix, evaluate_info)


def evaluate(prop_name, prop_value, conan_vars_info):
    """
    Evaluates a single prop_name, prop_value like "os", "Windows" against
    conan_vars_info.serialize_min()
    """

    def compatible_prop(setting_value, prop_value):
        return (prop_value == setting_value) or (prop_value == "None" and setting_value is None)

    info_settings = conan_vars_info.get("settings", [])
    info_options = conan_vars_info.get("options", [])

    if (prop_name in ["os", "os_build", "compiler", "arch", "arch_build", "build_type"] or
            prop_name.startswith("compiler.")):
        return compatible_prop(info_settings.get(prop_name, None), prop_value)
    else:
        return compatible_prop(info_options.get(prop_name, None), prop_value)
    return False


def search_recipes(paths, pattern=None, ignorecase=True):
    # Conan references in main storage
    if pattern:
        if isinstance(pattern, ConanFileReference):
            pattern = str(pattern)
        pattern = translate(pattern)
        pattern = re.compile(pattern, re.IGNORECASE) if ignorecase else re.compile(pattern)

    subdirs = list_folder_subdirs(basedir=paths.store, level=4)
    if not pattern:
        return sorted([ConanFileReference(*folder.split("/")) for folder in subdirs])
    else:
        ret = []
        for subdir in subdirs:
            ref = ConanFileReference(*subdir.split("/"))
            if _partial_match(pattern, ref):
                ret.append(ref)

        return sorted(ret)


def _partial_match(pattern, ref):
    """
    Finds if pattern matches any of partial sums of tokens of conan reference
    """
    tokens = str(ref).replace('/', ' / ').replace('@', ' @ ').split()

    def partial_sums(iterable):
        partial = ''
        for i in iterable:
            partial += i
            yield partial

    return any(map(pattern.match, list(partial_sums(tokens))))


def search_packages(cache, ref, query):
    """ Return a dict like this:

            {package_ID: {name: "OpenCV",
                           version: "2.14",
                           settings: {os: Windows}}}
    param ref: ConanFileReference object
    """
    if not os.path.exists(cache.conan(ref)):
        raise NotFoundException("Recipe not found: %s" % str(ref))
    infos = _get_local_infos_min(cache, ref)
    return filter_packages(query, infos)


def _get_local_infos_min(cache, ref):
    result = {}
    packages_path = cache.packages(ref)
    subdirs = list_folder_subdirs(packages_path, level=1)
    for package_id in subdirs:
        # Read conaninfo
        try:
            pref = PackageReference(ref, package_id)
            info_path = os.path.join(cache.package(pref, short_paths=None), CONANINFO)
            if not os.path.exists(info_path):
                raise NotFoundException("")
            conan_info_content = load(info_path)

            metadata = cache.package_layout(pref.ref).load_metadata()
            recipe_revision = metadata.packages[package_id].recipe_revision
            info = ConanInfo.loads(conan_info_content)
            if ref.revision and recipe_revision and recipe_revision != ref.revision:
                continue
            conan_vars_info = info.serialize_min()
            result[package_id] = conan_vars_info

        except Exception as exc:
            logger.error("Package %s has no ConanInfo file" % str(pref))
            if str(exc):
                logger.error(str(exc))
    return result



# coding=utf-8

import os

from conans.errors import ConanException
from conans.model.editable_cpp_info import EditableCppInfo
from conans.model.ref import ConanFileReference
from conans.model.ref import PackageReference
from conans.paths import CONANFILE


class PackageEditableLayout(object):

    def __init__(self, base_folder, layout_file, ref):
        assert isinstance(ref, ConanFileReference)
        self._ref = ref
        self._base_folder = base_folder
        self._layout_file = layout_file

    def conan(self):
        """ Returns the base folder for this package reference """
        return self._base_folder

    def conanfile(self):
        """ Path to the conanfile. We can agree that an editable package
            needs to be a Conan package
        """
        return os.path.join(self.conan(), CONANFILE)

    def editable_cpp_info(self):
        if self._layout_file:
            if os.path.isfile(self._layout_file):
                return EditableCppInfo.load(self._layout_file)
            else:
                raise ConanException("Layout file not found: %s" % self._layout_file)

    def export(self):
        raise ConanException("Operation not allowed on a package installed as editable")

    def export_sources(self):
        raise ConanException("Operation not allowed on a package installed as editable")

    def source(self):
        raise ConanException("Operation not allowed on a package installed as editable")

    def load_metadata(self):
        raise ConanException("Operation not allowed on a package installed as editable")

    def package(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref
        raise ConanException("Operation not allowed on a package installed as editable")

    def package_metadata(self):
        raise ConanException("Package metadata is not available for editable packages")



import unittest

from conans.test.utils.tools import TestClient


class OptimizeConanFileLoadTest(unittest.TestCase):

    def test_multiple_load(self):
        """ when a conanfile is used more than once in a dependency graph, the python file
        should be read and interpreted just once, then instance 2 different ConanFile
        objects. The module global value "mycounter" is global to all instances, this
        should be discouraged to use as if it was an instance value.
        In this test there are 2 nodes "Build/0.1" as it is a build-requires of both the
        conanfile.py and the test_package/conanfile.py
        """
        client = TestClient()
        conanfile = """from conans import ConanFile
mycounter = 0
class Pkg(ConanFile):
    mycounter2 = 0
    def configure(self):
        global mycounter
        mycounter += 1
        self.mycounter2 += 1
        self.output.info("MyCounter1 %s, MyCounter2 %s" % (mycounter, self.mycounter2))
"""
        client.save({"conanfile.py": conanfile})

        client.run("create . Build/0.1@user/testing")

        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": conanfile + "    def test(self): pass",
                     "myprofile": "[build_requires]\nBuild/0.1@user/testing"})

        client.run("create . Pkg/0.1@user/testing -pr=myprofile")
        self.assertIn("Build/0.1@user/testing: MyCounter1 2, MyCounter2 1", client.out)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest
from mock import mock

from conans.client.tools import OSInfo, environment_append, CYGWIN, MSYS2, MSYS, remove_from_path
from conans.errors import ConanException


class OSInfoTest(unittest.TestCase):

    def setUp(self):
        self._uname = None
        self._version = None

    def subprocess_check_output_mock(self, cmd, shell):
        if cmd.endswith('"uname"'):
            return self._uname.encode()
        elif cmd.endswith('"uname -r"'):
            return self._version.encode()
        raise ValueError("don't know how to respond to %s" % cmd)

    def test_windows(self):
        self._uname = None
        self._version = None
        with mock.patch("platform.system", mock.MagicMock(return_value='Windows')):
            with mock.patch.object(OSInfo, "get_win_version_name", return_value="Windows 98"):
                with mock.patch.object(OSInfo, "get_win_os_version", return_value="4.0"):
                    with remove_from_path("uname"):
                        with remove_from_path("bash"):
                            # FIXME: bug in environment_append removing variables which aren't exist (another PR)
                            new_env = dict()
                            if 'CONAN_BASH_PATH' in os.environ:
                                new_env['CONAN_BASH_PATH'] = None
                            with environment_append(new_env):
                                self.assertTrue(OSInfo().is_windows)
                                self.assertFalse(OSInfo().is_cygwin)
                                self.assertFalse(OSInfo().is_msys)
                                self.assertFalse(OSInfo().is_linux)
                                self.assertFalse(OSInfo().is_freebsd)
                                self.assertFalse(OSInfo().is_macos)
                                self.assertFalse(OSInfo().is_solaris)

                                with self.assertRaises(ConanException):
                                    OSInfo.uname()
                                self.assertIsNone(OSInfo.detect_windows_subsystem())

    def test_cygwin(self):
        self._uname = 'CYGWIN_NT-10.0'
        self._version = '2.11.2(0.329/5/3)'
        with mock.patch("platform.system", mock.MagicMock(return_value=self._uname)):
            with mock.patch.object(OSInfo, "get_win_version_name", return_value="Windows 98"):
                with mock.patch.object(OSInfo, "get_win_os_version", return_value="4.0"):
                    self.assertTrue(OSInfo().is_windows)
                    self.assertTrue(OSInfo().is_cygwin)
                    self.assertFalse(OSInfo().is_msys)
                    self.assertFalse(OSInfo().is_linux)
                    self.assertFalse(OSInfo().is_freebsd)
                    self.assertFalse(OSInfo().is_macos)
                    self.assertFalse(OSInfo().is_solaris)

                    with environment_append({"CONAN_BASH_PATH": "/fake/bash.exe"}):
                        with mock.patch('subprocess.check_output', new=self.subprocess_check_output_mock):
                            self.assertEqual(OSInfo.uname(), self._uname.lower())
                            self.assertEqual(OSInfo.detect_windows_subsystem(), CYGWIN)

    def test_msys2(self):
        self._uname = 'MSYS_NT-10.0'
        self._version = '1.0.18(0.48/3/2)'
        with mock.patch("platform.system", mock.MagicMock(return_value=self._uname)):
            with mock.patch.object(OSInfo, "get_win_version_name", return_value="Windows 98"):
                with mock.patch.object(OSInfo, "get_win_os_version", return_value="4.0"):
                    self.assertTrue(OSInfo().is_windows)
                    self.assertFalse(OSInfo().is_cygwin)
                    self.assertTrue(OSInfo().is_msys)
                    self.assertFalse(OSInfo().is_linux)
                    self.assertFalse(OSInfo().is_freebsd)
                    self.assertFalse(OSInfo().is_macos)
                    self.assertFalse(OSInfo().is_solaris)

                    with environment_append({"CONAN_BASH_PATH": "/fake/bash.exe"}):
                        with mock.patch('subprocess.check_output', new=self.subprocess_check_output_mock):
                            self.assertEqual(OSInfo.uname(), self._uname.lower())
                            self.assertEqual(OSInfo.detect_windows_subsystem(), MSYS)

    def test_mingw32(self):
        self._uname = 'MINGW32_NT-10.0'
        self._version = '2.10.0(0.325/5/3)'
        with mock.patch("platform.system", mock.MagicMock(return_value=self._uname)):
            with mock.patch.object(OSInfo, "get_win_version_name", return_value="Windows 98"):
                with mock.patch.object(OSInfo, "get_win_os_version", return_value="4.0"):
                    self.assertTrue(OSInfo().is_windows)
                    self.assertFalse(OSInfo().is_cygwin)
                    self.assertTrue(OSInfo().is_msys)
                    self.assertFalse(OSInfo().is_linux)
                    self.assertFalse(OSInfo().is_freebsd)
                    self.assertFalse(OSInfo().is_macos)
                    self.assertFalse(OSInfo().is_solaris)

                    with environment_append({"CONAN_BASH_PATH": "/fake/bash.exe"}):
                        with mock.patch('subprocess.check_output', new=self.subprocess_check_output_mock):
                            self.assertEqual(OSInfo.uname(), self._uname.lower())
                            self.assertEqual(OSInfo.detect_windows_subsystem(), MSYS2)

    def test_mingw64(self):
        self._uname = 'MINGW64_NT-10.0'
        self._version = '2.4.0(0.292/5/3)'
        with mock.patch("platform.system", mock.MagicMock(return_value=self._uname)):
            with mock.patch.object(OSInfo, "get_win_version_name", return_value="Windows 98"):
                with mock.patch.object(OSInfo, "get_win_os_version", return_value="4.0"):
                    self.assertTrue(OSInfo().is_windows)
                    self.assertFalse(OSInfo().is_cygwin)
                    self.assertTrue(OSInfo().is_msys)
                    self.assertFalse(OSInfo().is_linux)
                    self.assertFalse(OSInfo().is_freebsd)
                    self.assertFalse(OSInfo().is_macos)
                    self.assertFalse(OSInfo().is_solaris)

                    with environment_append({"CONAN_BASH_PATH": "/fake/bash.exe"}):
                        with mock.patch('subprocess.check_output', new=self.subprocess_check_output_mock):
                            self.assertEqual(OSInfo.uname(), self._uname.lower())
                            self.assertEqual(OSInfo.detect_windows_subsystem(), MSYS2)

    def test_linux(self):
        with mock.patch("platform.system", mock.MagicMock(return_value='Linux')):
            with mock.patch.object(OSInfo, '_get_linux_distro_info'):
                self.assertFalse(OSInfo().is_windows)
                self.assertFalse(OSInfo().is_cygwin)
                self.assertFalse(OSInfo().is_msys)
                self.assertTrue(OSInfo().is_linux)
                self.assertFalse(OSInfo().is_freebsd)
                self.assertFalse(OSInfo().is_macos)
                self.assertFalse(OSInfo().is_solaris)

                with self.assertRaises(ConanException):
                    OSInfo.uname()
                self.assertIsNone(OSInfo.detect_windows_subsystem())

    def test_macos(self):
        with mock.patch("platform.system", mock.MagicMock(return_value='Darwin')):
            self.assertFalse(OSInfo().is_windows)
            self.assertFalse(OSInfo().is_cygwin)
            self.assertFalse(OSInfo().is_msys)
            self.assertFalse(OSInfo().is_linux)
            self.assertFalse(OSInfo().is_freebsd)
            self.assertTrue(OSInfo().is_macos)
            self.assertFalse(OSInfo().is_solaris)

            with self.assertRaises(ConanException):
                OSInfo.uname()
            self.assertIsNone(OSInfo.detect_windows_subsystem())

    def test_freebsd(self):
        with mock.patch("platform.system", mock.MagicMock(return_value='FreeBSD')):
            self.assertFalse(OSInfo().is_windows)
            self.assertFalse(OSInfo().is_cygwin)
            self.assertFalse(OSInfo().is_msys)
            self.assertFalse(OSInfo().is_linux)
            self.assertTrue(OSInfo().is_freebsd)
            self.assertFalse(OSInfo().is_macos)
            self.assertFalse(OSInfo().is_solaris)

            with self.assertRaises(ConanException):
                OSInfo.uname()
            self.assertIsNone(OSInfo.detect_windows_subsystem())

    def test_solaris(self):
        with mock.patch("platform.system", mock.MagicMock(return_value='SunOS')):
            self.assertFalse(OSInfo().is_windows)
            self.assertFalse(OSInfo().is_cygwin)
            self.assertFalse(OSInfo().is_msys)
            self.assertFalse(OSInfo().is_linux)
            self.assertFalse(OSInfo().is_freebsd)
            self.assertFalse(OSInfo().is_macos)
            self.assertTrue(OSInfo().is_solaris)

            with self.assertRaises(ConanException):
                OSInfo.uname()
            self.assertIsNone(OSInfo.detect_windows_subsystem())



import os
import shutil

from six.moves.urllib.parse import urlparse

from conans import load
from conans.client import tools
from conans.client.cache.remote_registry import load_registry_txt
from conans.client.tools import Git
from conans.client.tools.files import unzip
from conans.errors import ConanException
from conans.util.files import mkdir, rmdir, walk


def _hide_password(resource):
    """
    Hide password from url/file path

    :param resource: string with url or file path
    :return: resource with hidden password if present
    """
    password = urlparse(resource).password
    return resource.replace(password, "<hidden>") if password else resource


def _handle_remotes(cache, remote_file):
    # FIXME: Should we encourage to pass the remotes in json?
    remotes, _ = load_registry_txt(load(remote_file))
    registry = cache.registry
    registry.remotes.define(remotes)


def _handle_profiles(source_folder, target_folder, output):
    mkdir(target_folder)
    for root, _, files in walk(source_folder):
        relative_path = os.path.relpath(root, source_folder)
        if relative_path == ".":
            relative_path = ""
        for f in files:
            profile = os.path.join(relative_path, f)
            output.info("  - %s" % profile)
            shutil.copy(os.path.join(root, f), os.path.join(target_folder, profile))


def _process_git_repo(repo_url, cache, output, tmp_folder, verify_ssl, args=None):
    output.info("Trying to clone repo: %s" % repo_url)

    with tools.chdir(tmp_folder):
        try:
            args = args or ""
            git = Git(verify_ssl=verify_ssl, output=output)
            git.clone(repo_url, args=args)
            output.info("Repo cloned!")
        except Exception as e:
            raise ConanException("Can't clone repo: %s" % str(e))
    _process_folder(tmp_folder, cache, output)


def _process_zip_file(zippath, cache, output, tmp_folder, remove=False):
    unzip(zippath, tmp_folder, output=output)
    if remove:
        os.unlink(zippath)
    _process_folder(tmp_folder, cache, output)


def _handle_conan_conf(current_conan_conf, new_conan_conf_path):
    current_conan_conf.read(new_conan_conf_path)
    with open(current_conan_conf.filename, "w") as f:
        current_conan_conf.write(f)


def _process_folder(folder, cache, output):
    for root, dirs, files in walk(folder):
        for f in files:
            if f == "settings.yml":
                output.info("Installing settings.yml")
                settings_path = cache.settings_path
                shutil.copy(os.path.join(root, f), settings_path)
            elif f == "conan.conf":
                output.info("Processing conan.conf")
                conan_conf = cache.conan_config
                _handle_conan_conf(conan_conf, os.path.join(root, f))
            elif f == "remotes.txt":
                output.info("Defining remotes from remotes.txt")
                _handle_remotes(cache, os.path.join(root, f))
            elif f == "README.md":
                output.info("Skip README.md")
            elif f == "LICENSE.txt":
                output.info("Skip LICENSE.txt")
            else:
                relpath = os.path.relpath(root, folder)
                target_folder = os.path.join(cache.conan_folder, relpath)
                mkdir(target_folder)
                output.info("Copying file %s to %s" % (f, target_folder))
                shutil.copy(os.path.join(root, f), target_folder)
        for d in dirs:
            if d == "profiles":
                output.info("Installing profiles:")
                profiles_path = cache.profiles_path
                _handle_profiles(os.path.join(root, d), profiles_path, output)
            elif d == "hooks" and ".git" not in root:  # Avoid git hooks
                output.info("Installing hooks:")
                src_hooks_path = os.path.join(root, d)
                dst_hooks_path = cache.hooks_path
                _handle_hooks(src_hooks_path, dst_hooks_path, output)
        dirs[:] = [d for d in dirs if d not in ("profiles", ".git", "hooks")]


def _process_download(item, cache, output, tmp_folder, verify_ssl, requester):
    output.info("Trying to download  %s" % _hide_password(item))
    zippath = os.path.join(tmp_folder, "config.zip")
    try:
        tools.download(item, zippath, out=output, verify=verify_ssl, requester=requester)
        _process_zip_file(zippath, cache, output, tmp_folder, remove=True)
    except Exception as e:
        raise ConanException("Error while installing config from %s\n%s" % (item, str(e)))


def configuration_install(path_or_url, cache, output, verify_ssl, requester, config_type=None,
                          args=None):
    if path_or_url is None:
        try:
            item = cache.conan_config.get_item("general.config_install")
            _config_type, path_or_url, _verify_ssl, _args = _process_config_install_item(item)
        except ConanException:
            raise ConanException("Called config install without arguments and "
                                 "'general.config_install' not defined in conan.conf")
    else:
        _config_type, path_or_url, _verify_ssl, _args = _process_config_install_item(path_or_url)

    config_type = config_type or _config_type
    verify_ssl = verify_ssl or _verify_ssl
    args = args or _args

    if os.path.exists(path_or_url):
        path_or_url = os.path.abspath(path_or_url)

    tmp_folder = os.path.join(cache.conan_folder, "tmp_config_install")
    # necessary for Mac OSX, where the temp folders in /var/ are symlinks to /private/var/
    tmp_folder = os.path.realpath(tmp_folder)
    mkdir(tmp_folder)
    try:

        if config_type == "git":
            _process_git_repo(path_or_url, cache, output, tmp_folder, verify_ssl, args)
        elif config_type == "dir":
            args = None
            _process_folder(path_or_url, cache, output)
        elif config_type == "file":
            args = None
            _process_zip_file(path_or_url, cache, output, tmp_folder)
        elif config_type == "url":
            args = None
            _process_download(path_or_url, cache, output, tmp_folder, verify_ssl,
                              requester=requester)
        else:
            raise ConanException("Unable to process config install: %s" % path_or_url)
    finally:
        if config_type is not None and path_or_url is not None:
            value = "%s, %s, %s, %s" % (config_type, path_or_url, verify_ssl, args)
            cache.conan_config.set_item("general.config_install", value)
        rmdir(tmp_folder)


def _process_config_install_item(item):
    """
    Processes a config_install item and outputs a tuple with the configuration type, the path/url
    and additional args

    :param item: config_install item value from conan.conf with a simple path/url or a string
    following this pattern: "<config_type>:[<path_or_url>, <args>]"
    :return: configuration source type (git, url, dir or file), path to file/dir or git/http url and
    additional arguments
    """
    config_type, path_or_url, verify_ssl, args = None, None, None, None
    if not item.startswith(("git,", "dir,", "url,", "file,")):
        path_or_url = item
        if path_or_url.endswith(".git"):
            config_type = "git"
        elif os.path.isdir(path_or_url):
            config_type = "dir"
        elif os.path.isfile(path_or_url):
            config_type = "file"
        elif path_or_url.startswith("http"):
            config_type = "url"
        else:
            raise ConanException("Unable to process config install: %s" % path_or_url)
    else:
        config_type, path_or_url, verify_ssl, args = [r.strip() for r in item.split(",")]
        verify_ssl = "true" in verify_ssl.lower()
        args = None if "none" in args.lower() else args
    return config_type, path_or_url, verify_ssl, args


def _handle_hooks(src_hooks_path, dst_hooks_path, output):
    """
    Copies files to the hooks folder overwriting the files that are in the same path
    (shutil.copytree fails on doing this), skips git related files (.git, .gitmodule...) and outputs
    the copied files

    :param src_hooks_path: Folder where the hooks come from
    :param dst_hooks_path:  Folder where the hooks should finally go
    :param output: Output to indicate the files copied
    """
    for root, dirs, files in walk(src_hooks_path):
        if ".git" in root:
            continue
        relpath = os.path.relpath(root, src_hooks_path)
        for f in files:
            if ".git" not in f:
                dst = os.path.join(dst_hooks_path, relpath)
                mkdir(dst)
                shutil.copy(os.path.join(root, f), dst)
                output.info(" - %s" % relpath)
